@@ -1,4 +1,4 @@
-__version__ = '0.35.0'
+__version__ = '0.36.0'
 
 from typing import TypedDict as _TypedDict
 from functools import partial as _partial
@@ -12,7 +12,7 @@ from jdatetime import datetime as _jdatetime
 from pandas import read_csv as _read_csv, DataFrame as _DataFrame
 # noinspection PyUnresolvedReferences
 from pandas import to_numeric as _to_numeric, read_html as _read_html
-from requests import get as _get
+from requests import get as _requests_get
 
 
 _csv2df = _partial(_read_csv, low_memory=False, engine='c', lineterminator=';')
@@ -81,13 +81,37 @@ def _parse_market_state(s: str) -> _MarketState:
     return result
 
 
-def _get_content(url: str) -> bytes:
-    return _get(url).content
+def _parse_ombud_messages(text) -> _DataFrame:
+    headers = _findall(r'<th>(.+?)</th>', text)
+    dates = _findall(r"<th class='ltr'>(.+?)</th>", text)
+    descriptions = _findall(r'<td colspan="2">(.+?)<hr />\s*</td>', text)
+    df = _DF({
+        'header': headers, 'date': dates, 'description': descriptions
+    }, dtype='string')
+    if dates:  # pandas cannot do ('14' + df['date']) on empty dates
+        df['date'] = ('14' + df['date']).apply(
+            _jstrptime, format='%Y/%m/%d %H:%M')
+    else:
+        df['date'] = df['date'].astype(object)
+    return df
 
 
 _FARSI_NORM = ''.maketrans('يك', 'یک')
 
 
-def _fa_norm_text(url: str) -> str:
-    # replace Arabic [ي ك] with Persian [ی ک]
-    return _get_content(url).decode().translate(_FARSI_NORM)
+def _get(url: str, *, fa=False) -> str | bytes:
+    result = _requests_get(url).content
+    if fa is True:
+        return result.decode().translate(_FARSI_NORM)
+    return result
+
+
+_DOMAIN = 'http://tsetmc.com/'
+
+
+def _get_data(path: str, *, fa=False) -> str | bytes:
+    return _get(f'{_DOMAIN}tsev2/data/' + path, fa=fa)
+
+
+def _get_par_tree(path: str, *, fa=True) -> str | bytes:
+    return _get(f'{_DOMAIN}Loader.aspx?ParTree={path}', fa=fa)
