@@ -1,8 +1,8 @@
 from numpy import nan as _nan
 
-from . import _parse_market_state, _TypedDict, _MarketState, _csv2df, \
-    _fa_norm_text, _StringIO, _get_content, _BytesIO, _DF, _DataFrame, \
-    _to_numeric, _read_html, _findall, _jstrptime
+from . import _get_data, _get_par_tree, _parse_market_state, _TypedDict, \
+    _MarketState, _csv2df, _StringIO, _BytesIO, _DF, _DataFrame, \
+    _parse_ombud_messages, _to_numeric, _read_html, _jstrptime
 
 
 _PRICE_INDEX_COLS = ['ins_code', 'isin', 'l18', 'l30']
@@ -27,7 +27,7 @@ _PRICE_DTYPES = {
     'visitcount': 'uint64',
     # 0-7 /dev/docs/Instrument.html
     'flow': 'uint8',
-    # 1-98 /dev/docs/cs_table.html
+    # 1-98, see tsetmc.general.cs_codes()
     'cs': 'uint8',
     'tmax': 'float64',
     'tmin': 'float64',
@@ -61,7 +61,8 @@ def market_watch_init(
         For `heven` see: /dev/docs/IndexB1LastDayOneInst.html
             (it's the time of the last transaction in HHMMSS format)
     """
-    text = _fa_norm_text('http://tsetmc.com/tsev2/data/MarketWatchInit.aspx?h=0&r=0')
+    # todo: use content?
+    text = _get_data('MarketWatchInit.aspx?h=0&r=0', fa=True)
     _, market_state_str, states, price_rows, _ = text.split('@')
     result = {}
     if prices:
@@ -104,9 +105,10 @@ def market_watch_plus(
 ) -> _MarketWatchPlus:
     # See dev/tsetmc_source_files/market_watch.html
     # for how the response is parsed in the browser.
-    handle_msg, update_fast_view, inst_price, best_limit, refid = _fa_norm_text(
-        'http://www.tsetmc.com/tsev2/data/MarketWatchPlus.aspx?'
-        f'h={5 * (heven // 5)}&r={25 * (refid // 25)}').split('@')
+    handle_msg, update_fast_view, inst_price, best_limit, refid = _get_data(
+        f'MarketWatchPlus.aspx?h={5 * (heven // 5)}&r={25 * (refid // 25)}',
+        fa=True
+    ).split('@')
     result = {}
     if messages:
         # whenever a new id appears, users should try to fetch new messages
@@ -163,7 +165,7 @@ def closing_price_all() -> _DataFrame:
         tsetmc filters. See:
             http://tsetmc.com/Site.aspx?ParTree=151715&LnkIdn=3197
     """
-    content = _get_content('http://www.tsetmc.com/tsev2/data/ClosingPriceAll.aspx')
+    content = _get_data('ClosingPriceAll.aspx')
     data = _split_id_rows(content, id_row_len=11)
     # dtype='uint64' param cannot be used due to
     # https://github.com/pandas-dev/pandas/issues/44835
@@ -179,7 +181,7 @@ def client_type_all() -> _DataFrame:
 
     In column names `n_` prefix stands for natural and `l_` for legal.
     """
-    content = _get_content('http://www.tsetmc.com/tsev2/data/ClientTypeAll.aspx')
+    content = _get_data('ClientTypeAll.aspx')
     df = _csv2df(
         _BytesIO(content), names=(
             'ins_code', 'n_buy_count', 'l_buy_count', 'n_buy_volume', 'l_buy_volume'
@@ -195,7 +197,7 @@ def key_stats() -> _DataFrame:
         http://www.tsetmc.com/Site.aspx?ParTree=151715&LnkIdn=3199 or
         http://cdn.tsetmc.com/Site.aspx?ParTree=151713
     """
-    content = _get_content('http://www.tsetmc.com/tsev2/data/InstValue.aspx?t=a')
+    content = _get_data('InstValue.aspx?t=a')
     data = _split_id_rows(content, id_row_len=3)
     df = _DF(data, columns=('ins_code', 'n', 'value'))
     # noinspection PyTypeChecker
@@ -206,24 +208,32 @@ def key_stats() -> _DataFrame:
 
 
 def ombud_messages(
-    top: int | str, flow: int | str = 0,
-):
-    text = _fa_norm_text(
-        'http://www.tsetmc.com/Loader.aspx?Partree=151313'
-        f'&Flow={flow}&top={top}')
-    headers = _findall(r'<th>(.+?)</th>', text)
-    dates = _findall(r"<th class='ltr'>(.+?)</th>", text)
-    descriptions = _findall(r'<td colspan="2">(.+?)<hr /></td>', text)
-    df = _DF({'header': headers, 'date': dates, 'description': descriptions})
-    # _jdatetime.strptime does not support %y directive
-    # https://github.com/slashmili/python-jalali/issues/100
-    df['date'] = ('14' + df['date']).apply(_jstrptime, format='%Y/%m/%d %H:%M')
-    return df
+    *, top: int | str = None, flow: int | str = None,
+    containing: str = None, sh_date: str = None,
+) -> _DataFrame:
+    """Return ombud messages as a dataframe.
+
+    :param top: How many messages to return.
+    :param flow: Only return messages in the given market flow.
+    :param containing: Only return messages containing this term.
+    :param sh_date: Solar Hijri date string in 'YYYY-mm-dd' format.
+
+    Note: the server ignores the `top` when `deven_persian` is given.
+    """
+    path = '151313'
+    if flow is not None:
+        path += f'&Flow={flow}'
+    if top is not None:
+        path += f'&top={top}'
+    if sh_date is not None:
+        path += f'&DevenPersian={sh_date}'
+    if containing is not None:
+        path += f'&Lval18AFC={containing}'
+    return _parse_ombud_messages(_get_par_tree(path))
 
 
 def status_changes(top: int | str) -> _DataFrame:
-    text = _fa_norm_text(
-        f'http://redirectcdn.tsetmc.com/Loader.aspx?Partree=15131L&top={top}')
+    text = _get_par_tree(f'15131L&top={top}')
     df = _read_html(text)[0]
     df['date'] = (df['تاریخ'] + ' ' + df['زمان']).apply(
         _jstrptime, format='%Y/%m/%d %H:%M')
